package mocks;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;

import tools.FinishedExecutionListener;

import asct.shared.ApplicationState;
import asct.shared.ExecutionRequestStatus;
import asct.shared.IExecutionListener;
import asct.shared.ParametricCopyHolder;
import asct.ui.ASCTController;
import asct.ui.ExecutionRequestDescriptorComposite;
import clusterManagement.ApplicationNotFoundException;
import clusterManagement.ApplicationRegistrationException;
import clusterManagement.BinaryCreationException;
import clusterManagement.DirectoryCreationException;
import clusterManagement.DirectoryNotFoundException;
import clusterManagement.InvalidPathNameException;
import clusterManagement.SecurityException;
import dataTypes.ApplicationType;
import dataTypes.ExecutionRequestId;

public class ASCTControllerMock extends ASCTController {

	private boolean haveListener;
	public String registeredApplicationBasePath;
	public String registeredApplicationName;
	public String uploadedBinaryBasePath;
	public String uploadedBinaryApplicationName;
	public String uploadedBinaryAbsolutePath;
	public String uploadedPlatform;
	public String descriptionFilePath;
	public String createdDirectory;
	private FinishedExecutionListener listener;
	private boolean isValid;

	@Override
	public void createDirectory(String directoryPath)
			throws DirectoryCreationException, InvalidPathNameException,
			SecurityException {
		createdDirectory = directoryPath;
	}

	@Override
	public ExecutionRequestDescriptorComposite readExecutionRequestDescriptorFile(
			BufferedReader descFile) throws FileNotFoundException, IOException {
		ExecutionRequestDescriptorComposite executionDescriptor = new ExecutionRequestDescriptorComposite();
		executionDescriptor.setApplicationName("Foo");
		executionDescriptor.setApplicationType(ApplicationType.regular);
		executionDescriptor.setApplicationBasePath("basePath");
		executionDescriptor.setApplicationBinaries("Quantic");
		return executionDescriptor;
	}

	@Override
	public void registerApplication(String basePath, String applicationName)
			throws ApplicationRegistrationException,
			DirectoryCreationException, InvalidPathNameException,
			SecurityException {
		registeredApplicationBasePath = basePath;
		registeredApplicationName = applicationName;
	}

	@Override
	public ExecutionRequestStatus executeApplication(String basePath,
			String applicationName, String applicationArguments,
			String applicationContraints, String applicationPreferences,
			ApplicationType applicationType, String binaryNames,
			String[] inputFiles, String[] outputFiles,
			ParametricCopyHolder[] parametricCopies, int numberOfTasks,
			boolean forceDifferentMachines)
			throws ApplicationNotFoundException, DirectoryNotFoundException,
			InvalidPathNameException, SecurityException {
		// TODO Auto-generated method stub
		listener.updateStatus(new ExecutionRequestId("0", "1000"),
				ApplicationState.FINISHED);
		if (isValid) {
			return new ExecutionRequestStatus(applicationName,
					ApplicationState.FINISHED, "0");
		} else {
			return null;
		}
	}

	@Override
	public void getApplicationResults(String execReqId, String[] outputFiles) {

	}

	@Override
	public void uploadBinary(String localFilePath, String remoteBasePath,
			String applicationName, String platform)
			throws BinaryCreationException, ApplicationNotFoundException,
			DirectoryNotFoundException, InvalidPathNameException,
			SecurityException, FileNotFoundException {
		uploadedBinaryAbsolutePath = localFilePath;
		uploadedBinaryBasePath = remoteBasePath;
		uploadedBinaryApplicationName = applicationName;
		uploadedPlatform = platform;
	}

	public ASCTControllerMock() {
		this(true);
	}

	public ASCTControllerMock(boolean valid) {
		haveListener = false;
		isValid = valid;
	}

	@Override
	public void registerExecutionStateListener(IExecutionListener listener) {
		this.listener = (FinishedExecutionListener) listener;
		haveListener = true;
	}

	public boolean receivedListener() {
		return haveListener;
	}

}
