package tools;

import java.io.IOException;

import junit.framework.Assert;
import mocks.ASCTControllerMock;

import org.junit.Before;
import org.junit.Test;

import asct.shared.ApplicationState;
import asct.shared.ExecutionRequestStatus;
import asct.ui.ASCTController;

public class CommonApplicationTestTest {

	private CommonApplicationTest application1;
	private ASCTController controller;
	private CommonApplicationTest application2;

	@Before
	public void setUp() {
		controller = ASCTController.getInstance();
		application1 = CommonApplicationTest.generate("ParametricTest",
				controller);
		application2 = CommonApplicationTest.generate("CudaTest", controller);
	}

	@Test
	public void shoulBeGeneratedWithAsctController() {
		Assert.assertEquals(controller, application1.getController());
		Assert.assertEquals(controller, application2.getController());
	}

	@Test
	public void shouldStoreItsName() {
		Assert.assertEquals("ParametricTest", application1.getName());
		Assert.assertEquals("CudaTest", application2.getName());
	}

	@Test
	public void shouldHaveSequencialIds() {
		application1 = CommonApplicationTest.generate("ParametricTest",
				controller);
		application2 = CommonApplicationTest.generate("CudaTest", controller);
		int offset = application1.getId();
		Assert.assertEquals(new Integer(offset + 1), application2.getId());
	}

	@Test
	public void shouldRegisterAListenerDuringCreations() {
		ASCTControllerMock controllerMock = new ASCTControllerMock();
		application1 = CommonApplicationTest.generate("FooApplication",
				(ASCTController) controllerMock);
		Assert.assertTrue(controllerMock.receivedListener());
	}

	@Test
	public void shouldSetupApplication() {
		ASCTControllerMock controllerMock = new ASCTControllerMock();
		application1 = CommonApplicationTest.generate("Foo",
				(ASCTController) controllerMock);
		try {
			execAndWait("/bin/mkdir -p /tmp/integrade_test/");
			execAndWait("/bin/touch /tmp/integrade_test/Foo.conf");
			application1.setup();
			execAndWait("/bin/rm -f /tmp/integrade_test/Foo.conf");
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			Assert.fail("An error ocurred in setup.");
		}

		Assert.assertEquals("basePath",
				controllerMock.registeredApplicationBasePath);
		Assert.assertEquals("Foo", controllerMock.registeredApplicationName);

		Assert.assertEquals("basePath", controllerMock.uploadedBinaryBasePath);
		Assert
				.assertEquals("Foo",
						controllerMock.uploadedBinaryApplicationName);
		Assert.assertEquals("/tmp/integrade_test/Foo",
				controllerMock.uploadedBinaryAbsolutePath);
		Assert.assertEquals("Quantic", controllerMock.uploadedPlatform);
		Assert.assertEquals("basePath", controllerMock.createdDirectory);

	}

	private void execAndWait(String command) throws InterruptedException,
			IOException {
		Runtime.getRuntime().exec(command).waitFor();
	}

	@Test
	public void shouldRunApplication() {
		ASCTControllerMock controllerMock = new ASCTControllerMock();
		application1 = CommonApplicationTest.generate("Foo",
				(ASCTController) controllerMock);

		try {
			execAndWait("/bin/mkdir -p /tmp/integrade_test/");
			execAndWait("/bin/touch /tmp/integrade_test/Foo.conf");
			application1.setup();
			execAndWait("/bin/rm -f /tmp/integrade_test/Foo.conf");
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			Assert.fail("An error ocurred in setup.");
		}

		ExecutionRequestStatus status = null;
		try {
			status = application1.run(1000);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			Assert.fail("An error ocurred in run.");
		}
		Assert.assertEquals("", ApplicationState.FINISHED.toString(), status
				.getApplicationState().toString());
	}

	@Test
	public void shouldAssertIfStatusIsNull() {
		ASCTControllerMock controllerMock = new ASCTControllerMock(false);
		application1 = CommonApplicationTest.generate("Foo",
				(ASCTController) controllerMock);

		try {
			execAndWait("/bin/mkdir -p /tmp/integrade_test/");
			execAndWait("/bin/touch /tmp/integrade_test/Foo.conf");
			application1.setup();
			execAndWait("/bin/rm -f /tmp/integrade_test/Foo.conf");
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			Assert.fail("An error ocurred in setup.");
		}

		ExecutionRequestStatus status = null;
		try {
			status = application1.run(1000);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			Assert.fail("An error ocurred in run.");
		}
		Assert.assertEquals(null, status);
	}
}
